/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Dmonstration du registre  dcalage 74HC595 avec 8 LEDs.
  Code source pour PIC12F683, dvelopp dans Microchip MPLAB X IDE v5.45 et compil avec Microchip XC8 v2.31.
  lectro-Bidouilleur, Aot 2021. https://www.youtube.com/lectro-Bidouilleur
  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr
 */

// INSTRUCTIONS AU COMPILATEUR
#include <xc.h>
#define _XTAL_FREQ 4000000                      // La frquence par dfaut de l'oscillateur interne

// OCTETS DE CONFIGURATION (FUSIBLES)
#pragma config FOSC = INTOSCIO                  // Bits de slection de l'oscillateur (oscillateur INTOSCIO: fonction E/S sur la broche RA4 / OSC2 / CLKOUT, fonction E/S sur RA5/OSC1/CLKIN)
#pragma config WDTE = OFF                       // Bit d'activation de l'horloge de surveillance (WDT dsactiv)
#pragma config PWRTE = OFF                      // Bit d'activation de la minuterie de mise sous tension (PWRT dsactiv)
#pragma config MCLRE = OFF                      // Bit de slection de fonction de broche MCLR (la fonction de broche MCLR configur en entre numrique, MCLR li en interne  VDD)
#pragma config CP = OFF                         // Bit de protection du code (la protection du code de la mmoire du programme est dsactive)
#pragma config CPD = OFF                        // Bit de protection du code de donnes (la protection du code de la mmoire de donnes est dsactive)
#pragma config BOREN = OFF                      // Brown Out Detect (BOR dsactiv)
#pragma config IESO = OFF                       // Bit de commutation externe/interne (le mode de commutation externe interne est dsactiv)
#pragma config FCMEN = OFF                      // Bit activ du moniteur d'horloge de scurit (le moniteur d'horloge de scurit est dsactiv) 

// VARIABLES
unsigned char sortie;
unsigned char i;
unsigned char j;

  // La squence de LEDs  envoyer est sauvegarde en mmoire flash de programme, pas dans la RAM 
const unsigned char sequence_LED[] =   {0b10000001,0b01000010,0b00100100,0b00011000,
                                        0b00011000,0b00100100,0b01000010,0b10000001,
                                        0b10000000,0b01000000,0b00100000,0b00010000,
                                        0b00001000,0b00000100,0b00000010,0b00000001,
                                        0b00000001,0b00000011,0b00000111,0b00001111,
                                        0b00011111,0b00111111,0b01111111,0b11111111};

// FONCTION PRINCIPALE
void main(void) 
{
	TRISIO 		=	0b11111000;                 // Assignation en entre ou sortie du port GPIO:    
                                                //	 5-Entre/Sortie disponible
    // Note: Les pullups internes sont          //	 4-Entre/Sortie disponible
    //       actives par dfaut                //	 3-Entre uniquement, disponible
                                                //	 2-Sortie de l'horloge des bascules de sortie (vers RCLK du 74HC595)
                                                //	 1-Sortie de l'horloge des bascule  dcalage (vers SRCLK du 74HC595)
                                                //	 0-Sortie de la donne srie (vers SER du 74HC595)
    while(1)  // Boucle principale
    {    
        for (i=0;i<sizeof(sequence_LED);i++)    // Boucle pour lire les squences des LEDs, une  la fois
        {
            sortie = sequence_LED[i];           // Sauvegarde de la squence dans une variable temporaire
            for (j=0;j<8;j++)                   // Boucle pour lire les 8 bits de la squence
            {
                GPIO = (GPIO & 0b11111110) | (sortie & 0b10000000)>>7;  // Pousse la valeur du bit 7 (MSB) de la squence vers la sortie GPIO-0
                GPIO = GPIO | 0b00000010;       // Envoie un coup d'horloge de dcalage
                GPIO = GPIO & 0b11111101;       //     "
                sortie = sortie<<1;             // Dcale la squence par un bit vers la gauche
            }
            GPIO = GPIO | 0b00000100;           // Envoie un coup d'horloge de sortie
            GPIO = GPIO & 0b11111011;           //     "
            __delay_ms(100);                    // Pause pour le visuel
        }
    }
}